/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/

#define BOOST_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <algorithm>

#include <boost/test/unit_test.hpp>
#include <boost/asio/io_service.hpp>
#include <boost/asio/ip/address.hpp>

#include "host/pingprotocol.h"
#include "dns/hostaddress.h"
#include "dns/dnsmaster.h"
#include "dns/dnscache.h"
#include "dns/resolverbase.h"

//------------------------------------------------------------------------------
// helper functions
//------------------------------------------------------------------------------

void create_master();

// TODO: try to run this only once for test suite
void create_master()
{
    // only create if it does not exist yet
    DnsMasterItem master = DnsMaster::get_instance();
    if ( !master )
    {
        boost::asio::ip::address name_server =
                             boost::asio::ip::address::from_string("127.0.0.1");
        int resolved_ip_ttl_threshold = 3;
        int max_address_resolution_attempts = 2;
        std::string cache_file = DO_NOT_USE_CACHE_FILE;
        IoServiceItem io_serv;

        // create it
        DnsMaster::create_master(io_serv,
                                 name_server,
                                 resolved_ip_ttl_threshold,
                                 max_address_resolution_attempts,
                                 cache_file);
        master = DnsMaster::get_instance();

        // simple checks
        BOOST_CHECK_EQUAL( master->get_resolved_ip_ttl_threshold(),
                                       resolved_ip_ttl_threshold );
        BOOST_CHECK_EQUAL( master->get_max_address_resolution_attempts(),
                                       max_address_resolution_attempts );
    }
}

//------------------------------------------------------------------------------
// test suite
//------------------------------------------------------------------------------

BOOST_AUTO_TEST_SUITE( TestDns )

BOOST_AUTO_TEST_CASE( create_master )
{
    create_master();
}

BOOST_AUTO_TEST_CASE( create_v4 )
{
    create_master();

    // create resolver
    std::string hostname = "www.intra2net.com";
    ResolverItem resolver = DnsMaster::get_instance()
                                ->get_resolver_for(hostname, PingProtocol_ICMP);
    BOOST_CHECK_EQUAL( resolver->get_hostname(), hostname );
    BOOST_CHECK( !resolver->is_resolving() );

    // cancel should have no effect
    resolver->cancel_resolve();
    BOOST_CHECK( !resolver->is_resolving() );

    // should not have any ips since cache did not load anything
    BOOST_CHECK_EQUAL( resolver->get_resolved_ip_count(), 0 );
    BOOST_CHECK( !resolver->have_up_to_date_ip() );
    std::string no_ip = "0.0.0.0";
    BOOST_CHECK_EQUAL( resolver->get_next_ip().get_ip().to_string(), no_ip );
}

BOOST_AUTO_TEST_SUITE_END()
